"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getHighlighter = getHighlighter;
exports.getSafeLanguage = getSafeLanguage;
exports.isValidLanguage = isValidLanguage;
var _core = require("@shikijs/core");
var _css = _interopRequireDefault(require("shiki/dist/langs/css.mjs"));
var _html = _interopRequireDefault(require("shiki/dist/langs/html.mjs"));
var _javascript = _interopRequireDefault(require("shiki/dist/langs/javascript.mjs"));
var _json = _interopRequireDefault(require("shiki/dist/langs/json.mjs"));
var _markdown = _interopRequireDefault(require("shiki/dist/langs/markdown.mjs"));
var _php = _interopRequireDefault(require("shiki/dist/langs/php.mjs"));
var _tsx = _interopRequireDefault(require("shiki/dist/langs/tsx.mjs"));
var _typescript = _interopRequireDefault(require("shiki/dist/langs/typescript.mjs"));
var _githubDark = _interopRequireDefault(require("shiki/dist/themes/github-dark.mjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable no-console */

// Define raw languages with proper typing
const rawLanguages = {
  typescript: _typescript.default,
  javascript: _javascript.default,
  json: _json.default,
  html: _html.default,
  css: _css.default,
  markdown: _markdown.default,
  tsx: _tsx.default,
  php: _php.default,
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    patterns: [{
      match: '.*',
      name: 'text.plain'
    }],
    repository: {}
  }
};
const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
let highlighterInstance = null;

// Create a simple regex engine that avoids lookbehind and lookahead assertions
function createSimpleRegexEngine() {
  return {
    createScanner: patterns => ({
      findNextMatchSync: (s, startPosition) => {
        for (let i = 0; i < patterns.length; i++) {
          try {
            // Remove lookbehind and lookahead assertions from the pattern
            const pattern = patterns[i];
            if (!pattern) continue;
            const cleanPattern = pattern.replace(/\(\?<[!=].*?\)/g, '') // Remove lookbehind
            .replace(/\(\?<=.*?\)/g, '') // Remove positive lookbehind
            .replace(/\(\?<!.*?\)/g, '') // Remove negative lookbehind
            .replace(/\(\?=.*?\)/g, '') // Remove positive lookahead
            .replace(/\(\?!.*?\)/g, ''); // Remove negative lookahead

            const regex = new RegExp(cleanPattern, 'g');
            regex.lastIndex = startPosition;
            const match = regex.exec(s);
            if (match) {
              return {
                index: match.index,
                matches: match,
                pattern: i,
                captureIndices: match.map((m, idx) => ({
                  start: match.index,
                  end: match.index + m.length,
                  length: m.length,
                  index: idx
                }))
              };
            }
          } catch (e) {
            console.warn('Invalid regex pattern:', patterns[i], e);
          }
        }
        return null;
      }
    }),
    createString: s => ({
      content: s,
      toString: () => s,
      substring: (start, end) => s.substring(start, end),
      length: s.length
    })
  };
}

// Process raw languages into LanguageRegistrations
function processLanguages() {
  return Object.entries(rawLanguages).map(([key, rawLang]) => {
    const lang = Array.isArray(rawLang) ? rawLang[0] : rawLang;

    // Create simpler patterns that don't rely on lookbehind/lookahead

    const patterns = (lang.patterns || []).map(pattern => {
      if (typeof pattern === 'string') {
        return {
          match: pattern.replace(/\(\?<[!=].*?\)/g, '')
        };
      }
      if (pattern.match) {
        pattern.match = pattern.match.replace(/\(\?<[!=].*?\)/g, '').replace(/\(\?<=.*?\)/g, '').replace(/\(\?<!.*?\)/g, '').replace(/\(\?=.*?\)/g, '').replace(/\(\?!.*?\)/g, '');
      }
      return pattern;
    });
    return {
      name: lang.name || key,
      scopeName: lang.scopeName || `source.${key}`,
      patterns,
      repository: lang.repository || {},
      aliases: lang.aliases || [],
      embeddedLangs: lang.embeddedLanguages || []
    };
  });
}
async function getHighlighter() {
  if (highlighterInstance) {
    return highlighterInstance;
  }
  try {
    // Create highlighter with simple regex engine
    highlighterInstance = await (0, _core.createHighlighterCore)({
      themes: [(0, _core.normalizeTheme)(_githubDark.default)],
      langs: processLanguages(),
      engine: createSimpleRegexEngine()
    });
    return highlighterInstance;
  } catch (error) {
    console.error('Error creating highlighter:', error);
    throw error;
  }
}
function isValidLanguage(lang) {
  const normalized = lang.toLowerCase();
  return Object.keys(rawLanguages).includes(normalized);
}
function getSafeLanguage(lang) {
  const normalized = lang.toLowerCase();
  if (isValidLanguage(normalized)) {
    return normalized;
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`);
  return 'plaintext';
}
//# sourceMappingURL=SyntaxContext.js.map