"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SyntaxHighlighter = void 0;
var _react = _interopRequireDefault(require("react"));
var _reactNative = require("react-native");
var _SyntaxContext = require("./SyntaxContext");
var _jsxRuntime = require("react/jsx-runtime");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable no-console */

const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[SyntaxHighlighter] ${message}`, ...args);
  }
}
const defaultHighlightLines = [];
const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines
}) => {
  const [loading, setLoading] = _react.default.useState(true);
  const [error, setError] = _react.default.useState(null);
  const [tokens, setTokens] = _react.default.useState([]);
  _react.default.useEffect(() => {
    const highlight = async () => {
      try {
        log('Starting highlighting process');
        log('Input:', {
          text: `${text.substring(0, 100)}...`,
          // Truncate for logging
          language,
          theme: theme.name,
          themeType: typeof theme
        });
        setLoading(true);
        const highlighter = await (0, _SyntaxContext.getHighlighter)();
        log('Highlighter instance obtained');

        // Debug highlighter state
        log('Highlighter capabilities:', {
          hasLanguage: Boolean(highlighter.getLoadedLanguages().includes(language)),
          loadedLanguages: highlighter.getLoadedLanguages()
        });
        if (!text || !language || !theme) {
          throw new Error('Missing required props');
        }

        // Verify language is loaded
        log('Checking language support...');
        const loadedLangs = highlighter.getLoadedLanguages();
        log('Loaded languages:', loadedLangs);
        if (!loadedLangs.includes(language)) {
          log('WARNING: Language not found in loaded languages!');
        }

        // Debug theme
        log('Theme details:', {
          name: theme.name,
          settings: theme.settings?.length || 'none',
          tokenColors: theme.tokenColors?.length || 'none'
        });
        log('Attempting tokenization...');
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme
        });
        log('Tokenization completed');
        log('Result structure:', {
          hasTokens: Boolean(result?.tokens),
          tokenCount: result?.tokens?.length || 0
        });
        if (result?.tokens) {
          log('Sample tokens:', result.tokens[0]?.slice(0, 2));
        }
        if (!result?.tokens) {
          throw new Error('Invalid tokenization result');
        }
        setTokens(result.tokens);
        setError(null);
      } catch (err) {
        const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code';
        log('Error during highlighting:', err);
        if (err instanceof Error) {
          log('Error stack:', err.stack);
        }
        log('State at error:', {
          textLength: text?.length,
          language,
          themeAvailable: Boolean(theme)
        });
        setError(errorMessage);
        setTokens([]);
      } finally {
        setLoading(false);
      }
    };
    highlight();
  }, [text, language, theme]);
  const renderToken = (token, index) => {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.Text, {
      style: [{
        color: token.color,
        fontSize,
        fontFamily: 'monospace',
        fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
        fontWeight: token.fontStyle === 1 ? 'bold' : 'normal'
      }, textStyle],
      children: token.content
    }, `${token.offset || index}-${index}`);
  };
  const renderLine = (lineTokens, index) => {
    const isHighlighted = highlightLines.includes(index + 1);
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      style: [{
        flexDirection: 'row',
        flexWrap: 'wrap',
        backgroundColor: isHighlighted ? 'rgba(255,255,255,0.1)' : 'transparent'
      }],
      children: lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
    }, index);
  };
  if (loading) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      style: [{
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.ActivityIndicator, {})
    });
  }
  if (error) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      style: [{
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/(0, _jsxRuntime.jsxs)(_reactNative.Text, {
        style: {
          color: '#ff6b6b'
        },
        children: ["Error:", error]
      })
    });
  }
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    style: [{
      backgroundColor: theme.colors?.background || '#1e1e1e'
    }, scrollViewProps?.style],
    contentContainerStyle: [{
      padding: 10
    }, containerStyle],
    children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      onStartShouldSetResponder: () => true,
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
};
exports.SyntaxHighlighter = SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map