/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration } from '@shikijs/types'
import { createHighlighterCore, normalizeTheme } from '@shikijs/core'

import css from 'shiki/dist/langs/css.mjs'
import html from 'shiki/dist/langs/html.mjs'
import javascript from 'shiki/dist/langs/javascript.mjs'
import json from 'shiki/dist/langs/json.mjs'
import markdown from 'shiki/dist/langs/markdown.mjs'
import php from 'shiki/dist/langs/php.mjs'
import tsx from 'shiki/dist/langs/tsx.mjs'

import typescript from 'shiki/dist/langs/typescript.mjs'
import githubDark from 'shiki/dist/themes/github-dark.mjs'

// Define raw languages with proper typing
const rawLanguages: Record<string, any> = {
  typescript,
  javascript,
  json,
  html,
  css,
  markdown,
  tsx,
  php,
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    patterns: [{ match: '.*', name: 'text.plain' }],
    repository: {},
  },
}

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args)
  }
}

export type AvailableLanguage = 'php' | 'tsx' | 'typescript' | 'javascript' | 'json' | 'html' | 'css' | 'markdown' | 'plaintext'

let highlighterInstance: HighlighterCore | null = null

// Create a simple regex engine that avoids lookbehind and lookahead assertions
function createSimpleRegexEngine() {
  return {
    createScanner: (patterns: string[]) => ({
      findNextMatchSync: (s: string, startPosition: number) => {
        for (let i = 0; i < patterns.length; i++) {
          try {
            // Remove lookbehind and lookahead assertions from the pattern
            const pattern = patterns[i]
            if (!pattern)
              continue

            const cleanPattern = pattern
              .replace(/\(\?<[!=].*?\)/g, '') // Remove lookbehind
              .replace(/\(\?<=.*?\)/g, '') // Remove positive lookbehind
              .replace(/\(\?<!.*?\)/g, '') // Remove negative lookbehind
              .replace(/\(\?=.*?\)/g, '') // Remove positive lookahead
              .replace(/\(\?!.*?\)/g, '') // Remove negative lookahead

            const regex = new RegExp(cleanPattern, 'g')
            regex.lastIndex = startPosition
            const match = regex.exec(s)
            if (match) {
              return {
                index: match.index,
                matches: match,
                pattern: i,
                captureIndices: match.map((m, idx) => ({ start: match.index, end: match.index + m.length, length: m.length, index: idx })),
              }
            }
          }
          catch (e) {
            console.warn('Invalid regex pattern:', patterns[i], e)
          }
        }
        return null
      },
    }),
    createString: (s: string) => ({
      content: s,
      toString: () => s,
      substring: (start: number, end?: number) => s.substring(start, end),
      length: s.length,
    }),
  }
}

// Process raw languages into LanguageRegistrations
function processLanguages(): LanguageRegistration[] {
  return Object.entries(rawLanguages).map(([key, rawLang]) => {
    const lang = Array.isArray(rawLang) ? rawLang[0] : rawLang

    // Create simpler patterns that don't rely on lookbehind/lookahead
    interface Pattern {
      match?: string
      [key: string]: any
    }

    const patterns: Pattern[] = (lang.patterns || []).map((pattern: string | Pattern): Pattern => {
      if (typeof pattern === 'string') {
        return { match: pattern.replace(/\(\?<[!=].*?\)/g, '') }
      }
      if (pattern.match) {
        pattern.match = pattern.match
          .replace(/\(\?<[!=].*?\)/g, '')
          .replace(/\(\?<=.*?\)/g, '')
          .replace(/\(\?<!.*?\)/g, '')
          .replace(/\(\?=.*?\)/g, '')
          .replace(/\(\?!.*?\)/g, '')
      }
      return pattern
    })

    return {
      name: lang.name || key,
      scopeName: lang.scopeName || `source.${key}`,
      patterns,
      repository: lang.repository || {},
      aliases: lang.aliases || [],
      embeddedLangs: lang.embeddedLanguages || [],
    }
  })
}

export async function getHighlighter() {
  if (highlighterInstance) {
    return highlighterInstance
  }

  try {
    // Create highlighter with simple regex engine
    highlighterInstance = await createHighlighterCore({
      themes: [normalizeTheme(githubDark)],
      langs: processLanguages(),
      engine: createSimpleRegexEngine(),
    })

    return highlighterInstance
  }
  catch (error) {
    console.error('Error creating highlighter:', error)
    throw error
  }
}

export function isValidLanguage(lang: string): lang is AvailableLanguage {
  const normalized = lang.toLowerCase()
  return Object.keys(rawLanguages).includes(normalized)
}

export function getSafeLanguage(lang: string): AvailableLanguage {
  const normalized = lang.toLowerCase()
  if (isValidLanguage(normalized)) {
    return normalized as AvailableLanguage
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`)
  return 'plaintext'
}
