/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, Text, type TextStyle, View, type ViewStyle } from 'react-native'
import { type AvailableLanguage, getHighlighter } from './SyntaxContext'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[SyntaxHighlighter] ${message}`, ...args)
  }
}

export interface SyntaxHighlighterProps {
  text: string
  language: AvailableLanguage
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  highlightLines?: number[]
}

const defaultHighlightLines: number[] = []

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines,
}) => {
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])

  React.useEffect(() => {
    const highlight = async () => {
      try {
        log('Starting highlighting process')
        log('Input:', {
          text: `${text.substring(0, 100)}...`, // Truncate for logging
          language,
          theme: theme.name,
          themeType: typeof theme,
        })

        setLoading(true)
        const highlighter = await getHighlighter()
        log('Highlighter instance obtained')

        // Debug highlighter state
        log('Highlighter capabilities:', {
          hasLanguage: Boolean(highlighter.getLoadedLanguages().includes(language)),
          loadedLanguages: highlighter.getLoadedLanguages(),
        })

        if (!text || !language || !theme) {
          throw new Error('Missing required props')
        }

        // Verify language is loaded
        log('Checking language support...')
        const loadedLangs = highlighter.getLoadedLanguages()
        log('Loaded languages:', loadedLangs)
        if (!loadedLangs.includes(language)) {
          log('WARNING: Language not found in loaded languages!')
        }

        // Debug theme
        log('Theme details:', {
          name: theme.name,
          settings: theme.settings?.length || 'none',
          tokenColors: theme.tokenColors?.length || 'none',
        })

        log('Attempting tokenization...')
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme,
        })

        log('Tokenization completed')
        log('Result structure:', {
          hasTokens: Boolean(result?.tokens),
          tokenCount: result?.tokens?.length || 0,
        })

        if (result?.tokens) {
          log('Sample tokens:', result.tokens[0]?.slice(0, 2))
        }

        if (!result?.tokens) {
          throw new Error('Invalid tokenization result')
        }

        setTokens(result.tokens)
        setError(null)
      }
      catch (err) {
        const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code'
        log('Error during highlighting:', err)
        if (err instanceof Error) {
          log('Error stack:', err.stack)
        }
        log('State at error:', {
          textLength: text?.length,
          language,
          themeAvailable: Boolean(theme),
        })
        setError(errorMessage)
        setTokens([])
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme])

  const renderToken = (token: ThemedToken, index: number) => {
    return (
      <Text
        key={`${token.offset || index}-${index}`}
        style={[
          {
            color: token.color,
            fontSize,
            fontFamily: 'monospace',
            fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
            fontWeight: token.fontStyle === 1 ? 'bold' : 'normal',
          },
          textStyle,
        ]}
      >
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => {
    const isHighlighted = highlightLines.includes(index + 1)
    return (
      <View
        key={index}
        style={[
          {
            flexDirection: 'row',
            flexWrap: 'wrap',
            backgroundColor: isHighlighted ? 'rgba(255,255,255,0.1)' : 'transparent',
          },
        ]}
      >
        {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
      </View>
    )
  }

  if (loading) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor: theme.colors?.background || '#1e1e1e' }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 10 }, containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}
